######################## R script: MBWinferenceExample #########################

# For computing confidence intervals for fixed effects parameters and standard
# deviation and correlation parameters from the random effects covariance matrix
# in logistic generalized linear mixed models with bivariate random effects. 
# Confidence intervals based on the theory of Maestrini, Bhaskaran & Wand are 
# obtained through the function TwoTermAsyCov() and compared with those from 
# Jiang, Wand & Bhaskaran (2022, JRSSB) produced using OneTermAsyCov() and those 
# from the 'glmmTMB' package. Graphical comparison of the confidence intervals 
# is performed via the function graphicCompareCIs().

# Last changed: 26 JUN 2023

# Load required package and functions:

library(glmmTMB)
library(MASS)
library(matrixcalc)
library(cubature)
library(Rcpp)
library(RcppArmadillo)
library(parallel)
library(doParallel)
library(stipkg)
source("generExampData.r")
source("OneTermAsyCov.r")
source("TwoTermAsyCov.r")
source("graphicCompareCIs.r")

# Set values corresponding to the specified example:

family <- "binomial"
m <- 150 ; nVec <- rep(15,m)
betaATrue <- c(0.35,0.96)
betaBTrue <- c(-0.47,1.06,-1.31)
dA <- length(betaATrue) ; dB <- length(betaBTrue)
SigmaTrue <- matrix(c(0.56,-0.34,-0.34,0.89),2,2)
XAdistn <- "unif" ; XBdistn <- "unif"

# Obtain data:

allData <- generExampData(m,nVec,XAdistn,XBdistn,betaATrue,betaBTrue,SigmaTrue,family)

# Obtain the maximimum likelihood fit using the glmmTMB::glmmTMB() function:

fitMLE <- glmmTMB(y ~ 1 + XA[,-1] + XB + (1 + XA[,-1]|idNum),data=allData,family=family)
betaMLEtable <- coefficients(summary(fitMLE))
betaMLE <- fitMLE$fit$par[1:(dA+dB)]
betaAMLE <- betaMLE[1:dA]
betaBMLE <- betaMLE[(dA+1):(dA+dB)]
SigmaMLE <- matrix((summary(fitMLE)$varcor$cond$idNum)[1:(dA^2)],dA,dA)

# Obtain the one-term asymptotic covariance for beta and Sigma based on the
# Jiang, Wand & Bhaskaran (2022) asymptotic normality theorem:

JWBres <- OneTermAsyCov(betaAMLE,betaBMLE,SigmaMLE,allData$XA,allData$XB)

# Obtain the two-term asymptotic covariance for beta and Sigma based on the
# Maestrini, Bhaskaran & Wand asymptotic results using a parallel backend:

cores <- detectCores()
clus <- makeCluster(cores[1]-1)
registerDoParallel(clus)
MBWres <- TwoTermAsyCov(betaAMLE,betaBMLE,SigmaMLE,allData$XA,allData$XB,m,nVec)
stopCluster(clus)

# Graphically compare confidence intervals from different approaches:

graphicCompareCIs(MBWres,JWBres,fitMLE,betaATrue,betaBTrue,SigmaTrue)

######################### End of MBWinferenceExample ###########################
