########################### R function: OneTermAsyCov ##########################

# For estimating the one-term asymptotic covariance of the beta parameter vector 
# and Sigma covariance matrix for logistic generalized linear mixed models,
# based on the asymptotic normality theorem of Jiang, Wand & Bhaskaran (2022, 
# JRSSB). 

# Last changed: 26 JUN 2023

OneTermAsyCov <- function(betaAhat,betaBhat,SigmaHat,XA,XB)
{
   dA <- ncol(XA) ; dB <- ncol(XB)
   
   # Set the bddLogis() and bdddLogis() functions:
   
   bddLogis <- function(x) return(0.5/(1 + cosh(x)))
   bdddLogis <- function(x) return(-0.5*sinh(x)/((1 + cosh(x))^2))
   
   # Create duplication and Moore-Penrose inverse matrices:
   
   DdA <- duplication.matrix(dA)
   DdAplus <- solve(crossprod(DdA),t(DdA))
   
   # Set the (k,kd) entry integrand function and its derivative:
   
   intgdLogisOneTerm <- function(uArg,idx1,idx2)
   {
      u <- uArg[1:dA]
      
      # Obtain useful vectors and matrices:
      
      XbetaAhatPu <- as.vector(crossprod(t(XA),(betaAhat+u)))
      XbetaBhat <- as.vector(crossprod(t(XB),betaBhat))
      bddVec <- bddLogis(XbetaAhatPu + XbetaBhat)
      
      XAXB <- cbind(XA,XB)
      LRinds <- ((dA+1):(dA+dB))
      Theta0 <- crossprod((XAXB*bddVec),XAXB)   
      
      retVal <- (solve(solve(Theta0)[LRinds,LRinds])[idx1,idx2]
                 *(2*pi)^(-0.5*dA)*(det(SigmaHat))^(-0.5)
                 *exp(-0.5*t(u)%*%solve(SigmaHat)%*%u))
      
      return(retVal)
   } 
   
   # Fill up the entries of invLambdaHatBetaB:
   
   invLambdaHatBetaB <- matrix(NA,dB,dB)
   for (k in 1:dB)
   {
      for (kd in k:dB) 
      {
         invLambdaHatBetaB[k,kd] <- hcubature(intgdLogisOneTerm,lowerLimit=rep(-Inf,dA),
                                             upperLimit=rep(Inf,dA),idx1=k,idx2=kd)$integral
         invLambdaHatBetaB[kd,k] <- invLambdaHatBetaB[k,kd]
      }
   }

   # Obtain the asymptotic covariance matrix of beta-hat:
   
   AsyCovBetaHat <- matrix(NA,(dA+dB),(dA+dB))
   AsyCovBetaHat[1:dA,1:dA] <- (1/m)*diag(SigmaHat)
   AsyCovBetaBhat <- solve(invLambdaHatBetaB)
   AsyCovBetaHat[(dA+1):(dA+dB),(dA+1):(dA+dB)] <- AsyCovBetaBhat
   
   # Obtain the asymptotic covariance matrix of Sigma-hat:
   
   AsyCovVechSigmaHat <- matrix(NA,dA*(dA + 1)/2,dA*(dA + 1)/2)
   AsyCovVechSigmaHat <- (2/m)*DdAplus%*%kronecker(SigmaTrue,SigmaTrue)%*%t(DdAplus)
   
   # Return the one-term asymptotic covariance:
   
   return(list(AsyCovBetaHat=AsyCovBetaHat,AsyCovVechSigmaHat=AsyCovVechSigmaHat))
}

############################# End of OneTermAsyCov #############################
 