########################### R function: TwoTermAsyCov ##########################

# For estimating the two-term asymptotic covariance of the beta_A and beta_B 
# parameter vectors and Sigma matrix for logistic generalized linear mixed 
# models based on the results of Maestrini, Bhaskaran & Wand. 

# Last changed: 26 JUN 2023

TwoTermAsyCov <- function(betaAhat,betaBhat,SigmaHat,XA,XB,m,nVec)
{
  # Load required functions:
  
  source("vecInverse.r")
  source("vechInverse.r")
  
  # Set integral limits:
  
  lowLim <- -Inf  ;  uppLim <- Inf
  
  # Obtain useful dimensional quantities:
  
  dA <- ncol(XA) ; dB <- ncol(XB)
  n <- mean(nVec) ; dR <- dA ; dF <- dB + dA  
  if (dR>=dF)
    stop("The length of the random effects parameter vector is not smaller 
          than the length of the fixed effects paramter vector.\n") 
  dAxdApOneHalf <- dA*(dA + 1)/2
  dBxdBpOneHalf <- dB*(dB + 1)/2   
  nUniqEPsi89 <- dAxdApOneHalf*(dAxdApOneHalf + 1)/2
  nUniqLambdaAB <- dA*dB
  nUniqDelta <- dB*dAxdApOneHalf
  dAxdAidx <- matrix(NA,dAxdApOneHalf,2)
  idx <- 1
  for (k in 1:dA) 
    for (kd in k:dA) 
    {
      dAxdAidx[idx,1] <- k
      dAxdAidx[idx,2] <- kd
      idx <- idx + 1
    }
  dBxdBidx <- matrix(NA,dBxdBpOneHalf,2)
  idx <- 1
  for (k in 1:dB) 
    for (kd in k:dB) 
    {
      dBxdBidx[idx,1] <- k
      dBxdBidx[idx,2] <- kd
      idx <- idx + 1
    }
  EPsi89idx <- matrix(NA,nUniqEPsi89,2)
  idx <- 1
  for (k in 1:dAxdApOneHalf) 
    for (kd in k:dAxdApOneHalf) 
    {
      EPsi89idx[idx,1] <- k
      EPsi89idx[idx,2] <- kd
      idx <- idx + 1
    }
  dAxdBidx <- matrix(NA,nUniqLambdaAB,2)
  idx <- 1
  for (kd in 1:dB) 
    for (k in 1:dA)
    {
      dAxdBidx[idx,1] <- k
      dAxdBidx[idx,2] <- kd
      idx <- idx + 1
    }
  DeltaIdx <- matrix(NA,nUniqDelta,2)
  idx <- 1
  for (kd in 1:dAxdApOneHalf) 
    for (k in 1:dB) 
    {
      DeltaIdx[idx,1] <- k
      DeltaIdx[idx,2] <- kd
      idx <- idx + 1
    }
  
  # Set b''() and b'''() functions for the logistic case:
  
  bddLogis <- function(x) return(0.5/(1 + cosh(x)))
  bdddLogis <- function(x) return(-0.5*sinh(x)/((1 + cosh(x))^2))

  # Compute beta_B^T%*%X_B vector:
  
  XbetaBhat <- as.vector(XB%*%betaBhat)
  
  # Create duplication and Moore-Penrose inverse matrices:
  
  DdR <- duplication.matrix(dR)
  DdRplus <- solve(crossprod(DdR),t(DdR))
  
  ##############################################################################
  # Define integrand functions: 
  
  intgLogis <- function(uArg,retOptn,rIdx,cIdx)
  {
    # Declare random effect vector u:
    u <- uArg[1:dA]
    
    # Obtain the needed vectors, matrices and arrays:
    XbetaAhatPu <- as.vector(XA%*%(betaAhat+u))
    bddVec <- bddLogis(XbetaAhatPu + XbetaBhat)
    bdddVec <- bdddLogis(XbetaAhatPu + XbetaBhat)
    OmegaAA <- (1/(m*n))*crossprod((XA*bddVec),XA)
    OmegaAAinv <- solve(OmegaAA)
    OmegaAB <- (1/(m*n))*crossprod((XA*bddVec),XB)
    OmegaBB <- (1/(m*n))*crossprod((XB*bddVec),XB)
    OmegaAAAdArray <- OmegaAAAd(bdddVec,XA,m*n,dR)
    OmegaAABdArray <- OmegaAABd(bdddVec,XA,XB,m*n,dR,dF)
    SigmaHatInv <- solve(SigmaHat)
    uuT <- tcrossprod(u)
    
    # Define random effect multivariate normal density:
    MNormDens <- ((2*pi)^(-0.5*dA)*(det(SigmaHat))^(-0.5)
                  *exp(-0.5*crossprod(u,SigmaHatInv)%*%u))
    
    # Obtain useful quantities for asymptotic results:
    psi1 <- vech(SigmaHat - uuT)
    psi2 <- as.vector(starProd(OmegaAAAdArray,OmegaAAinv,dR,dR,dR))
    psi3 <- as.vector(starProd(OmegaAABdArray,OmegaAAinv,dR,dR,(dF-dR)))
    psi4 <- DdRplus%*%vec(OmegaAAinv%*%SigmaHatInv%*%(SigmaHat - uuT 
                                               - SigmaHat%*%tcrossprod(psi2,u)))
    Psi5 <- OmegaAAinv%*%OmegaAB
    Psi6 <- OmegaBB - crossprod(Psi5,OmegaAB)
    if(retOptn=="Psi6") return(Psi6[rIdx,cIdx]*MNormDens)
    Psi7 <- uuT%*%SigmaHatInv%*%OmegaAAinv
    Psi8 <- DdRplus%*%tcrossprod(kronecker(uuT,OmegaAAinv),DdRplus)
    if(retOptn=="Psi8") return(Psi8[rIdx,cIdx]*MNormDens)
    Psi9 <- tcrossprod(psi1,psi4) + tcrossprod(psi4,psi1)
    if(retOptn=="Psi9") return(Psi9[rIdx,cIdx]*MNormDens)
    
    # Define matrix arguments of expectations:
    LambdaAAintgd <- (Psi7 + t(Psi7) + OmegaAAinv%*%tcrossprod(psi2,u) 
                      + u%*%crossprod(psi2,OmegaAAinv) - OmegaAAinv )
    if(retOptn=="LambdaAA") return(LambdaAAintgd[rIdx,cIdx]*MNormDens)
    LambdaABintgd <- (uuT%*%SigmaHatInv%*%Psi5 + u%*%crossprod(psi2,Psi5) 
                      - tcrossprod(u,psi3))
    if(retOptn=="LambdaAB") return(LambdaABintgd[rIdx,cIdx]*MNormDens)
    DeltaIntgd <- tcrossprod((crossprod(Psi5,(SigmaHatInv%*%u + psi2)) - psi3),
                           psi1)
    if(retOptn=="Delta") return(DeltaIntgd[rIdx,cIdx]*MNormDens)
  }
  ##############################################################################
  
  # Fill up the entries of E(Psi_6):
  
  EPsi6vec <- foreach(
    i = 1:dBxdBpOneHalf, 
    .combine = 'c',
    .packages = c("cubature","stipkg","matrixcalc")
  ) %dopar% {
    hcubature(intgLogis,lowerLimit=rep(lowLim,dA),upperLimit=rep(uppLim,dA),
              retOptn="Psi6",rIdx=dBxdBidx[i,1],cIdx=dBxdBidx[i,2])$integral
  }
  EPsi6 <- vechInverse(EPsi6vec)
  cat("E(Psi_6) computed;\n")

  # Fill up the entries of E(Psi_8):
  
  EPsi8vec <- foreach(
    i = 1:nUniqEPsi89,
    .combine = 'c',
    .packages = c("cubature","stipkg","matrixcalc")
  ) %dopar% {
    hcubature(intgLogis,lowerLimit=rep(lowLim,dA),upperLimit=rep(uppLim,dA),
              retOptn="Psi8",rIdx=EPsi89idx[i,1],cIdx=EPsi89idx[i,2])$integral
  }
  EPsi8 <- vechInverse(EPsi8vec)
  cat("E(Psi_8) computed;\n")
  
  # Fill up the entries of E(Psi_9):
  
  EPsi9vec <- foreach(
    i = 1:nUniqEPsi89,
    .combine = 'c',
    .packages = c("cubature","stipkg","matrixcalc")
  ) %dopar% {
    hcubature(intgLogis,lowerLimit=rep(lowLim,dA),upperLimit=rep(uppLim,dA),
              retOptn="Psi9",rIdx=EPsi89idx[i,1],cIdx=EPsi89idx[i,2])$integral
  }
  EPsi9 <- vechInverse(EPsi9vec)
  cat("E(Psi_9) computed;\n")
  
  # Fill up the entries of Lambda_{AA}:
  
  LambdaAAvec <- foreach(
    i = 1:dAxdApOneHalf,
    .combine = 'c',
    .packages = c("cubature","stipkg","matrixcalc")
  ) %dopar% {
    hcubature(intgLogis,lowerLimit=rep(lowLim,dA),upperLimit=rep(uppLim,dA),
              retOptn="LambdaAA",rIdx=dAxdAidx[i,1],cIdx=dAxdAidx[i,2])$integral
  }
  LambdaAA <- vechInverse(LambdaAAvec)
  cat("Lambda_{AA} computed;\n")
  
  # Fill up the entries of Lambda_{AB}:
  
  LambdaABvec <- foreach(
    i = 1:nUniqLambdaAB,
    .combine = 'c',
    .packages = c("cubature","stipkg","matrixcalc")
  ) %dopar% {
    hcubature(intgLogis,lowerLimit=rep(lowLim,dA),upperLimit=rep(uppLim,dA),
              retOptn="LambdaAB",rIdx=dAxdBidx[i,1],cIdx=dAxdBidx[i,2])$integral
  }
  LambdaAB <- vecInverse(LambdaABvec,c(dA,dB))
  cat("Lambda_{AB} computed;\n")
  
  # Fill up the entries of Delta:
  
  DeltaVec <- foreach(
    i = 1:nUniqDelta,
    .combine = 'c',
    .packages = c("cubature","stipkg","matrixcalc")
  ) %dopar% {
    hcubature(intgLogis,lowerLimit=rep(lowLim,dA),upperLimit=rep(uppLim,dA),
              retOptn="Delta",rIdx=DeltaIdx[i,1],cIdx=DeltaIdx[i,2])$integral
  }
  Delta <- vecInverse(DeltaVec,c(dB,dAxdApOneHalf))
  cat("Delta computed;\n")
  
  # Compute Cov(vech(beta-hat)|X):
  
  CovBetaHat <- CovBetaHat2ndTermInv <- matrix(NA,(dA+dB),(dA+dB))
  CovBetaHat2ndTermInv[1:dA,1:dA] <- solve(LambdaAA)
  CovBetaHat2ndTermInv[1:dA,(dA+1):(dA+dB)] <- solve(LambdaAA,LambdaAB)
  CovBetaHat2ndTermInv[(dA+1):(dA+dB),1:dA] <- t(CovBetaHat2ndTermInv[1:dA,(dA+1):(dA+dB)])
  CovBetaHat2ndTermInv[(dA+1):(dA+dB),(dA+1):(dA+dB)] <- (CovBetaHat2ndTermInv[(dA+1):(dA+dB),1:dA]%*%LambdaAB 
                                                          + EPsi6)
  CovBetaHat <- (1/(m*n))*solve(CovBetaHat2ndTermInv)
  CovBetaHat[1:dA,1:dA] <- SigmaHat/m + CovBetaHat[1:dA,1:dA]
  
  # Compute Cov(vech(Sigma-hat)|X):
  
  CovVechSigmaHat <- ((2/m)*DdRplus%*%kronecker(SigmaTrue,SigmaTrue)%*%t(DdRplus)
                      + (1/(m*n))*(2*EPsi9 - 4*EPsi8 + t(Delta)%*%solve(EPsi6,Delta)))
  
  # Return two-term asymptotic covariance:
  
  cat("Two-term asymptotic covariance computed.\n")
  return(list(CovBetaHat=CovBetaHat,CovVechSigmaHat=CovVechSigmaHat))
}

############################# End of TwoTermAsyCov #############################
