######################### R function: graphicCompareCIs ########################

# For plotting confidence intervals of fixed effects parameters and standard 
# deviation and correlation components from the random effects covariance 
# matrix. The confidence intervals obtained via the Maestrini, Bhaskaran & Wand
# theory are compared with those obtained using Theorem 1 of Jiang, Wand & 
# Bhaskaran (2022, JRSSB) and the 'glmmTMB' package.

# Last changed: 28 JUN 2023

graphicCompareCIs <- function(TwoTermRes,OneTermRes,fitMLE,
                              betaATrue,betaBTrue,SigmaTrue,alpha=0.05)
{
  # Load required functions:
  
  source("covParmsCIs1TermLogis.r")
  source("covParmsCIs2TermLogis.r")
  
  # Set plotting values:
  
  lwdVal <- 3
  colVec <- c("red","blue","green3")
  
  # Extract the fixed and random effects estimates, and random effects 
  # covariance based on maximum likelihood estimation:
  
  dA <- length(betaATrue) ; dB <- length(betaBTrue)
  betaMLE <- fitMLE$fit$par[1:(dA+dB)]
  SigmaMLE <- matrix((summary(fitMLE)$varcor$cond$idNum)[1:(dA^2)],dA,dA)
  
  # Compute confidence intervals from JWB theory:
  
  SDbetaJWB <- sqrt(diag(OneTermRes$AsyCovBetaHat))
  qVal <- 1 - alpha/2
  lowJWBbeta <- betaMLE - qnorm(qVal)*SDbetaJWB
  uppJWBbeta <- betaMLE + qnorm(qVal)*SDbetaJWB
  covParmsCIsJWB <- covParmsCIs1TermLogis(SigmaMLE,m,nVec[1])
  lowJWBcovParms <- c(covParmsCIsJWB$sigma1[1],covParmsCIsJWB$sigma2[1],
                      covParmsCIsJWB$rho[1])
  uppJWBcovParms <- c(covParmsCIsJWB$sigma1[2],covParmsCIsJWB$sigma2[2],
                      covParmsCIsJWB$rho[2])
  
  # Compute confidence intervals from MBW theory:
  
  SDbetaMBW <- sqrt(diag(TwoTermRes$CovBetaHat))
  AsyCovVechSigma2Term <- TwoTermRes$CovVechSigmaHat
  lowMBWbeta <- betaMLE - qnorm(qVal)*SDbetaMBW
  uppMBWbeta <- betaMLE + qnorm(qVal)*SDbetaMBW
  covParmsCIsMBW <- covParmsCIs2TermLogis(SigmaMLE,AsyCovVechSigma2Term,m,nVec[1])
  lowMBWcovParms <- c(covParmsCIsMBW$sigma1[1],covParmsCIsMBW$sigma2[1],
                      covParmsCIsMBW$rho[1])
  uppMBWcovParms <- c(covParmsCIsMBW$sigma1[2],covParmsCIsMBW$sigma2[2],
                      covParmsCIsMBW$rho[2])

  # Compute confidence intervals from glmmTMB:
  
  TMBconfint <- confint(fitMLE,method="Wald",level=1-alpha)
  lowTMBbeta <- TMBconfint[(1:(dA+dB)),1]
  uppTMBbeta <- TMBconfint[(1:(dA+dB)),2]
  lowTMBcovParms <- TMBconfint[(dA+dB+1):(dA+dB+dA*(dA+1)/2),1]
  uppTMBcovParms <- TMBconfint[(dA+dB+1):(dA+dB+dA*(dA+1)/2),2]
  
  # Set up plot vertical grid:

  numCIpars <- dA + dB + dA*(dA + 1)/2
  yGrid <- seq(0,1,length=(numCIpars + 1))[-1]
  
  # Set shift amount:
  
  shftAmt <- 0.1*(yGrid[2] - yGrid[1])
  
  # Plot each of the confidence intervals:
  
  xlimVal <- range(c(lowTMBbeta,uppTMBbeta,lowMBWbeta,uppMBWbeta,lowTMBcovParms,
                     uppTMBcovParms,lowMBWcovParms,uppMBWcovParms))
  
  xTextPos <- 1.1*xlimVal[1] - 0.1*xlimVal[2]
  xLineLow <- 1.05*xlimVal[1] - 0.05*xlimVal[2]
  
  plot(0,type="n",xlim=xlimVal,ylim=c(0,1.2),bty="l",yaxt="n",
       xlab="random and fixed effects parameters",ylab="")
  
  par(xpd=TRUE)
  
  betaTrue <- c(betaATrue,betaBTrue)
  covParmsTrue <- c(sqrt(diag(SigmaTrue)),
                    SigmaTrue[2,1]/(sqrt(SigmaTrue[1,1]*SigmaTrue[2,2])))
  
  for (iCI in 1:(dA+dB))
  {
    indxForYpos <- numCIpars - iCI + 1
    parmCurr <- bquote(beta[paste(.(iCI-1))])   
    text(xTextPos,yGrid[indxForYpos],parmCurr,cex=1.5)
    lines(c(xLineLow,xlimVal[2]),rep(yGrid[indxForYpos],2),col="grey",lty=2)
    lines(rep(betaTrue[iCI],2),
          c(yGrid[indxForYpos]-2*shftAmt,yGrid[indxForYpos]+2*shftAmt),
          col="black",lwd=0.5*lwdVal)  
    lines(c(lowJWBbeta[iCI],uppJWBbeta[iCI]),rep(yGrid[indxForYpos]+shftAmt,2),
          col=colVec[1],lwd=lwdVal) 
    lines(c(lowMBWbeta[iCI],uppMBWbeta[iCI]),rep(yGrid[indxForYpos],2),
          col=colVec[2],lwd=lwdVal)  
    lines(c(lowTMBbeta[iCI],uppTMBbeta[iCI]),rep(yGrid[indxForYpos]-shftAmt,2),
          col=colVec[3],lwd=lwdVal) 
  }
  
  for (iCI in (dA+dB+1):numCIpars)
  {
    indxForYpos <- numCIpars - iCI + 1
    
    if (iCI==(dA+dB+1)) parmCurr <- bquote(sigma[1])
    if (iCI==(dA+dB+2)) parmCurr <- bquote(sigma[2])
    if (iCI==(dA+dB+3)) parmCurr <- bquote(rho)
    
    text(xTextPos,yGrid[indxForYpos],parmCurr,cex=1.5)
    lines(c(xLineLow,xlimVal[2]),rep(yGrid[indxForYpos],2),col="grey",lty=2)
    lines(rep(covParmsTrue[iCI-(dA+dB)],2),
          c(yGrid[indxForYpos]-2*shftAmt,yGrid[indxForYpos]+2*shftAmt),
          col="black",lwd=0.5*lwdVal) 
    lines(c(lowJWBcovParms[iCI-(dA+dB)],uppJWBcovParms[iCI-(dA+dB)]),
          rep(yGrid[indxForYpos]+shftAmt,2),col=colVec[1],lwd=lwdVal) 
    lines(c(lowMBWcovParms[iCI-(dA+dB)],uppMBWcovParms[iCI-(dA+dB)]),
          rep(yGrid[indxForYpos],2),col=colVec[2],lwd=lwdVal) 
    lines(c(lowTMBcovParms[iCI-(dA+dB)],uppTMBcovParms[iCI-(dA+dB)]),
          rep(yGrid[indxForYpos]-shftAmt,2),col=colVec[3],lwd=lwdVal) 
  }
  
  # Add a legend:
  
  legend("topleft",legend=c("one-term asy. var.","two-term asy. var.",
         "R pack. glmmTMB","true value"),col=c(colVec,"black"),
         lty=c(rep(1,3),0),lwd=c(rep(lwdVal,3),0.5*lwdVal),bty="n",
         pch=c(rep("",3),"|"),cex=0.8)
  
  invisible()
}

########################### End of graphicCompareCIs ###########################
