############################# R script: powSSexamp #############################

# For doing an example of a power sample size computation involving a 
# Poisson-response mixed model with a random intercept and a random slope
# where dR=dF.

# Last changed: 26 OCT 2023


# Load required package:

library(MASS)
library(glmmTMB)

# Choose between one- or two-term asymptotic variance:

AsyVar <- "TwoTerm" #"OneTerm"

# Set model parameters:

beta0True <- -1.5
beta1True <- 0.3
SigmaTrue00 <- 0.5
SigmaTrue11 <- 0.25
SigmaTrue10 <- SigmaTrue01 <- 0.1
SigmaTrue <- matrix(c(SigmaTrue00,SigmaTrue10,SigmaTrue01,SigmaTrue11),2,2)
n <- 20

# Set other values corresponding to the specified example:

p <- 0.5
alpha <- 0.05
powVal <- 0.9

# Compute the one- or two-term asymptotic variance of beta_1^hat:

quadVal <- (qnorm(alpha) + qnorm(1 - powVal))^2
if(AsyVar=="OneTerm") cTerm <- 0
if(AsyVar=="TwoTerm")
{
  cTerm <- ((1/p)*exp(-beta0True-beta1True
                      + 0.5*(SigmaTrue00+2*SigmaTrue01+SigmaTrue11))
            + (1/(1-p))*exp(-beta0True+0.5*SigmaTrue00))
}

# Obtain minumum number of subjects m:

m <- ceiling((quadVal/(beta1True^2))*(SigmaTrue11+cTerm/n))

# Obtain the empirical power value:

numReps <- 100
numRejecs <- 0
for (irep in 1:numReps)
{
  set.seed(irep)
  
  # Generate data:
  
  u <- mvrnorm(m,rep(0,2),SigmaTrue)

  y <- NULL ; xVec <- NULL ; idNum <- NULL
  
  for (i in 1:m)
  {
    xCurr <- rbinom(n,1,p)
    muCurr <- exp(as.vector(beta0True + u[i,1] + (beta1True + u[i,2])*xCurr)) 
    yCurr <- rpois(n,muCurr)
    xVec <- c(xVec,xCurr)
    y <- c(y,yCurr)
    idNum <- c(idNum,rep(i,n))
  }
  
  # Fit the linear mixed model:
  
  fitObj <- glmmTMB(y ~ 1 + xVec + (1 + xVec|idNum),family="poisson")
  
  # Extract the t statistic value and check for hypothesis test rejection:
  
  summaryCoeffObj <- summary(fitObj)$coefficients$cond
  zVal <- summaryCoeffObj[2,3]
  cutOffVal <- qnorm(1 - alpha)
  if(zVal>cutOffVal) numRejecs <- numRejecs + 1 
  
  if(irep%%10==0) 
    cat("Replicate number ",irep," of ",numReps," completed.\n",sep="")
}

# Print the simulation experiment results:

Phat <- numRejecs/numReps
PhatRound <- round(Phat,3)
SEval <- sqrt(Phat*(1-Phat)/numReps)
CIlow <- Phat - qnorm(0.975)*SEval
CIupp <- Phat + qnorm(0.975)*SEval
CI <- round(c(CIlow,CIupp),3)
cat("The minimum number of subjects (m) is:",m,".\n")
cat("The empirical power is:",PhatRound,".\n")
cat("A 95% confidence interval for the empirical power is: (",CI,") .\n")

############################### End of powSSexamp ##############################
